# -*- coding: utf-8 -*-
# Import Jamendo new-style dump
# Copyright (C) 2010  Sylvain Beucler
#
# This file is part of Djamengo.
# 
# Djamengo is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
# 
# Djamengo is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
# Author: Sylvain Beucler

import djamengo.models as my_models
from xml.etree.ElementTree import iterparse
import gzip
import sys
import datetime
from dateutil import parser as dateutil_parser
import codecs

#f = gzip.GzipFile("dbdump_test.xml.gz", "r")
f = gzip.GzipFile("dbdump_artistalbumtrack.xml.gz", "r")

# get an iterable
context = iterparse(f, events=('start', 'end'))
# turn it into an iterator
#context = iter(context)
# get the root element
#event, root = context.next()

#out = codecs.open('out.sql', 'w', encoding="UTF-8")
outs = {}

def sql_copy(prefix, table, values):
    if table not in outs:
        outs[table] = codecs.open('%02d-%s.sql' % (prefix, table), 'w', encoding="UTF-8")
        outs[table].write("COPY %s (%s) FROM STDIN;\n" % (table, ",".join([name for (name, value) in values])))
    first = True
    for (name, value) in values:
        if type(value) == unicode or type(value) == str:
            value = value.replace("\\", "\\\\") \
                .replace("\t", "\\t").replace("\n", "\\n").replace("\r", "\\r")
        if value is None:
            value = "\\N"
        if not first:
            outs[table].write("\t")
        first = False
        outs[table].write(unicode(value))
    outs[table].write("\n")

nb = 0
root_node=None
artists_node=None
registered_tags={}
registered_tags_lastid = 0
for event, elem in context:
    if event == 'start' and elem.tag == 'JamendoData':
        root_node = elem
        sys.stderr.write("Dump date: %s\n" % datetime.datetime.fromtimestamp(int(root_node.get('epoch'))))
    if event == 'start' and elem.tag == 'Artists':
        artists_node = elem
    if event == 'end' and elem.tag == 'artist':
        location = elem.find('location')
        location_country=None
        location_state=None
        location_city=None
        location_latitude=None
        location_longitude=None
        if location:
            location_country=location.findtext('country')
            location_state=location.findtext('state')
            location_city=location.findtext('city')
            if location.findtext('latitude'):
                try:
                    location_latitude=float(location.findtext('latitude'))
                except ValueError:
                    pass
            if location.findtext('longitude'):
                try:
                    location_longitude=float(location.findtext('longitude'))
                except ValueError:
                    pass
        artist_id = int(elem.findtext('id'))
        sql_copy(1, "djamengo_artist", (('id', artist_id),
                                        ('name', elem.findtext('name')),
                                        ('url', elem.findtext('url')),
                                        ('image', elem.findtext('image')),
                                        ('mbgid', elem.findtext('mbgid')),
                                        ('location_country', location_country),
                                        ('location_state', location_state),
                                        ('location_city', location_city),
                                        ('location_latitude', location_latitude),
                                        ('location_longitude', location_longitude),
                                        ))
        for album in elem.find('Albums').findall('album'):
            id3genre = None
            if album.findtext('id3genre'):
                id3genre = int(album.findtext('id3genre'))
            album_id = int(album.findtext('id'))
            sql_copy(2, "djamengo_album", (('id', album_id),
                                           ('artist_id', artist_id),
                                           ('name', album.findtext('name')),
                                           ('url', album.findtext('url')),
                                           ('id3genre', id3genre),
                                           ('mbgid', album.findtext('mbgid')),
                                           ('license_artwork', album.findtext('license_artwork')),
                                           ('releasedate', dateutil_parser.parse(album.findtext('releasedate'))),
                                           ('filename', album.findtext('filename')),
                                        ))
            for track in album.find('Tracks').findall('track'):
                id3genre = None
                if track.findtext('id3genre'):
                    id3genre = int(track.findtext('id3genre'))
                track_id = int(track.findtext('id'))
                sql_copy(3, "djamengo_track", (('id', track_id),
                                               ('album_id', album_id),
                                               ('name', track.findtext('name')),
                                               ('filename', track.findtext('filename')),
                                               ('mbgid', track.findtext('mbgid')),
                                               ('numalbum', int(track.findtext('numalbum'))),
                                               ('id3genre', id3genre),
                                               ('license', track.findtext('license')),
                                               ('duration', float(track.findtext('duration'))),
                                               ))
                tags = []
                if track.find('Tags'):
                    tags = track.find('Tags').findall('tag')
                for tag in tags:
                    tag_id = None
                    tag_idstr = tag.findtext('idstr')
                    if tag_idstr in registered_tags:
                        tag_id = registered_tags[tag_idstr]
                    else:
                        registered_tags_lastid += 1
                        tag_id = registered_tags_lastid
                        registered_tags[tag_idstr] = tag_id
                        sql_copy(4, "djamengo_tag", (('id', tag_id),
                                                     ('idstr', tag_idstr),
                                                     ))
                    sql_copy(5, "djamengo_tracktag", (('track_id', track_id),
                                                      ('tag_id', tag_id),
                                                      ('weight', float(tag.findtext('weight'))),
                                                      ))
                    tag.clear()
                track.clear()
            album.clear()
        elem.clear()
        artists_node.clear()
        root_node.clear()

        # Print progress
        nb += 1
        if nb % 1000 == 0:
            sys.stderr.write("%d\n" % nb)

for f in outs:
    outs[f].write("\\.\n")
    outs[f].close()
